@@ -61,6 +61,7 @@ struct _GstRTSPMediaFactoryPrivate
   GstRTSPAddressPool *pool;
   GstRTSPTransportMode transport_mode;
   gboolean stop_on_disconnect;
+  gchar *multicast_iface;
 
   GstClockTime rtx_time;
   guint latency;
@@ -71,6 +72,8 @@ struct _GstRTSPMediaFactoryPrivate
   GType media_gtype;
 
   GstClock *clock;
+
+  GstRTSPPublishClockMode publish_clock_mode;
 };
 
 #define DEFAULT_LAUNCH          NULL
@@ -260,6 +263,7 @@ gst_rtsp_media_factory_init (GstRTSPMediaFactory * factory)
   priv->latency = DEFAULT_LATENCY;
   priv->transport_mode = DEFAULT_TRANSPORT_MODE;
   priv->stop_on_disconnect = DEFAULT_STOP_ON_DISCONNECT;
+  priv->publish_clock_mode = GST_RTSP_PUBLISH_CLOCK_MODE_CLOCK;
 
   g_mutex_init (&priv->lock);
   g_mutex_init (&priv->medias_lock);
@@ -282,6 +286,7 @@ gst_rtsp_media_factory_finalize (GObject * obj)
   g_mutex_clear (&priv->lock);
   if (priv->pool)
     g_object_unref (priv->pool);
+  g_free (priv->multicast_iface);
 
   G_OBJECT_CLASS (gst_rtsp_media_factory_parent_class)->finalize (obj);
 }
@@ -796,6 +801,64 @@ gst_rtsp_media_factory_get_address_pool (GstRTSPMediaFactory * factory)
   return result;
 }
 
+/**
+ * gst_rtsp_media_factory_set_multicast_iface:
+ * @media_factory: a #GstRTSPMediaFactory
+ * @multicast_iface: (transfer none): a multicast interface name
+ *
+ * configure @multicast_iface to be used for @media_factory.
+ */
+void
+gst_rtsp_media_factory_set_multicast_iface (GstRTSPMediaFactory * media_factory,
+    const gchar * multicast_iface)
+{
+  GstRTSPMediaFactoryPrivate *priv;
+  gchar *old;
+
+  g_return_if_fail (GST_IS_RTSP_MEDIA_FACTORY (media_factory));
+
+  priv = media_factory->priv;
+
+  GST_LOG_OBJECT (media_factory, "set multicast interface %s", multicast_iface);
+
+  g_mutex_lock (&priv->lock);
+  if ((old = priv->multicast_iface) != multicast_iface)
+    priv->multicast_iface = multicast_iface ? g_strdup (multicast_iface) : NULL;
+  else
+    old = NULL;
+  g_mutex_unlock (&priv->lock);
+
+  if (old)
+    g_free (old);
+}
+
+/**
+ * gst_rtsp_media_factory_get_multicast_iface:
+ * @media_factory: a #GstRTSPMediaFactory
+ *
+ * Get the multicast interface used for @media_factory.
+ *
+ * Returns: (transfer full): the multicast interface for @media_factory. g_free() after
+ * usage.
+ */
+gchar *
+gst_rtsp_media_factory_get_multicast_iface (GstRTSPMediaFactory * media_factory)
+{
+  GstRTSPMediaFactoryPrivate *priv;
+  gchar *result;
+
+  g_return_val_if_fail (GST_IS_RTSP_MEDIA_FACTORY (media_factory), NULL);
+
+  priv = media_factory->priv;
+
+  g_mutex_lock (&priv->lock);
+  if ((result = priv->multicast_iface))
+    result = g_strdup (result);
+  g_mutex_unlock (&priv->lock);
+
+  return result;
+}
+
 /**
  * gst_rtsp_media_factory_set_profiles:
  * @factory: a #GstRTSPMediaFactory
@@ -1275,6 +1338,51 @@ gst_rtsp_media_factory_get_clock (GstRTSPMediaFactory * factory)
   return ret;
 }
 
+/**
+ * gst_rtsp_media_factory_set_publish_clock_mode:
+ * @factory: a #GstRTSPMediaFactory
+ * @mode: the clock publish mode
+ *
+ * Sets if and how the media clock should be published according to RFC7273.
+ *
+ * Since: 1.8
+ */
+void
+gst_rtsp_media_factory_set_publish_clock_mode (GstRTSPMediaFactory * factory,
+    GstRTSPPublishClockMode mode)
+{
+  GstRTSPMediaFactoryPrivate *priv;
+
+  GST_RTSP_MEDIA_FACTORY_LOCK (factory);
+  priv = factory->priv;
+  priv->publish_clock_mode = mode;
+  GST_RTSP_MEDIA_FACTORY_UNLOCK (factory);
+}
+
+/**
+ * gst_rtsp_media_factory_get_publish_clock_mode:
+ * @factory: a #GstRTSPMediaFactory
+ *
+ * Gets if and how the media clock should be published according to RFC7273.
+ *
+ * Returns: The GstRTSPPublishClockMode
+ *
+ * Since: 1.8
+ */
+GstRTSPPublishClockMode
+gst_rtsp_media_factory_get_publish_clock_mode (GstRTSPMediaFactory * factory)
+{
+  GstRTSPMediaFactoryPrivate *priv;
+  GstRTSPPublishClockMode ret;
+
+  GST_RTSP_MEDIA_FACTORY_LOCK (factory);
+  priv = factory->priv;
+  ret = priv->publish_clock_mode;
+  GST_RTSP_MEDIA_FACTORY_UNLOCK (factory);
+
+  return ret;
+}
+
 static gchar *
 default_gen_key (GstRTSPMediaFactory * factory, const GstRTSPUrl * url)
 {
@@ -1419,6 +1527,8 @@ default_configure (GstRTSPMediaFactory * factory, GstRTSPMedia * media)
   guint latency;
   GstRTSPTransportMode transport_mode;
   GstClock *clock;
+  gchar *multicast_iface;
+  GstRTSPPublishClockMode publish_clock_mode;
 
   /* configure the sharedness */
   GST_RTSP_MEDIA_FACTORY_LOCK (factory);
@@ -1433,6 +1543,7 @@ default_configure (GstRTSPMediaFactory * factory, GstRTSPMedia * media)
   transport_mode = priv->transport_mode;
   stop_on_disconnect = priv->stop_on_disconnect;
   clock = priv->clock ? gst_object_ref (priv->clock) : NULL;
+  publish_clock_mode = priv->publish_clock_mode;
   GST_RTSP_MEDIA_FACTORY_UNLOCK (factory);
 
   gst_rtsp_media_set_suspend_mode (media, suspend_mode);
@@ -1445,6 +1556,7 @@ default_configure (GstRTSPMediaFactory * factory, GstRTSPMedia * media)
   gst_rtsp_media_set_latency (media, latency);
   gst_rtsp_media_set_transport_mode (media, transport_mode);
   gst_rtsp_media_set_stop_on_disconnect (media, stop_on_disconnect);
+  gst_rtsp_media_set_publish_clock_mode (media, publish_clock_mode);
 
   if (clock) {
     gst_rtsp_media_set_clock (media, clock);
@@ -1455,6 +1567,10 @@ default_configure (GstRTSPMediaFactory * factory, GstRTSPMedia * media)
     gst_rtsp_media_set_address_pool (media, pool);
     g_object_unref (pool);
   }
+  if ((multicast_iface = gst_rtsp_media_factory_get_multicast_iface (factory))) {
+    gst_rtsp_media_set_multicast_iface (media, multicast_iface);
+    g_free (multicast_iface);
+  }
   if ((perms = gst_rtsp_media_factory_get_permissions (factory))) {
     gst_rtsp_media_set_permissions (media, perms);
     gst_rtsp_permissions_unref (perms);
